/**
 * Copyright (C) 2014 Roman Hiestand
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this software
 * and associated documentation files (the "Software"), to deal in the Software without restriction,
 * including without limitation the rights to use, copy, modify, merge, publish, distribute,
 * sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all copies or substantial
 * portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
 * LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#ifndef PJDIRTREETHREAD_H
#define PJDIRTREETHREAD_H

/**
 * This class recursively searches a directory tree for JPG or PJG files in a worker thread.
 *
 */
class PJDirTreeThread : public wxThread
{
public:

	class FoundFile
	{
	public:
		FoundFile() : inFilename(wxT("")), outFilename(wxT("")) {}
		FoundFile(const FoundFile &o) :
			inFilename(o.inFilename), outFilename(o.outFilename)
		{}
		FoundFile &operator=(const FoundFile &o)
		{
			inFilename = o.inFilename; outFilename = o.outFilename;
			return *this;
		}

		wxString inFilename, outFilename;
	};

	PJDirTreeThread();
	virtual ~PJDirTreeThread();

	enum PJDTTJobType
	{
		PJDTTJobTypeCompress = 0,
		PJDTTJobTypeDecompress
	};

	bool startJob(PJDTTJobType jobType, const wxString &path);
	bool getWorkFile(FoundFile &file);

protected:
	virtual wxThread::ExitCode Entry();

	void traverseDir(const wxFileName &path, const wxFileName &outpath);
	void addWorkFile(const FoundFile &file);

private:
	PJDTTJobType jobType_;
	wxString path_;

	wxMutex mutex_;
	std::list< FoundFile > workFiles_;
};

#endif
