/**
 * Copyright (C) 2014 Roman Hiestand
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this software
 * and associated documentation files (the "Software"), to deal in the Software without restriction,
 * including without limitation the rights to use, copy, modify, merge, publish, distribute,
 * sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all copies or substantial
 * portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
 * LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#include "CommonIncludes.h"
#include "PJFileType.h"


PJFileType::PJFileType()
{
}

PJFileType::~PJFileType()
{
}

/**
 * Determine file type by examining the magic bytes at the beginning of the file.
 *
 * JPEG:	FF D8
 * PJG:		4A 53 ('JS')
 */
PJFileType::FileType PJFileType::determineFileType(const wxString &filename)
{
	FileType retVal = FileTypeUnknown;
	wxFFile testfile(filename);
	if(testfile.IsOpened())
	{
		unsigned char testbytes[2];
		size_t numBytes = testfile.Read(testbytes, 2);
		if(numBytes == 2)
		{
			if(testbytes[0] == 0xFF && testbytes[1] == 0xD8)
				retVal = FileTypeJPEG;
			else if(testbytes[0] == 0x4A && testbytes[1] == 0x53)
				retVal = FileTypePJG;
		}
		testfile.Close();
	}
	return retVal;
}
