/**
 * Copyright (C) 2014 Roman Hiestand
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this software
 * and associated documentation files (the "Software"), to deal in the Software without restriction,
 * including without limitation the rights to use, copy, modify, merge, publish, distribute,
 * sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all copies or substantial
 * portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
 * LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#ifndef PJWORKERTHREADMANAGER_H
#define PJWORKERTHREADMANAGER_H

class PJWorkerThread;

class PJWorkerThreadManager
{
public:

	class WorkItem
	{
	public:
		WorkItem() : inFilename(wxT("")), outFilename(wxT("")), processMsg(wxT("")), isOK(true) {}
		WorkItem(const WorkItem &o) :
			inFilename(o.inFilename), outFilename(o.outFilename),
			processMsg(o.processMsg), isOK(o.isOK)
		{}
		WorkItem &operator=(const WorkItem &o)
		{
			inFilename = o.inFilename; outFilename = o.outFilename;
			processMsg = o.processMsg; isOK = o.isOK;
			return *this;
		}

		wxString inFilename, outFilename, processMsg;
		bool isOK;
	};

	PJWorkerThreadManager();
	virtual ~PJWorkerThreadManager();

	// Called from main thread
	void clearWorkItems();
	void addWorkItem(wxString inFilename, wxString outFilename);
	bool existsOutfilename(const wxString &outFilename);
	void setParameters(bool verify, bool ignoreWarnings, bool discardMetadata, bool createDir);
	void startWorking();
	double getProgress();
	bool isFinishedWorking();
	void stopWorking();
	bool getIsWorking() { return isWorking_; }
	bool readFinishedWorkItem(wxString &inFilename, wxString &outFilename, wxString &processMsg, bool &isOK);
	void cancelWorking();

	// Called from worker threads
	bool getWorkItem(wxString &inFilename, wxString &outFilename);
	void finishedWorkItem(wxString inFilename, wxString outFilename, wxString processMsg, bool isOK);

private:

	wxMutex mutex_;
	std::list<WorkItem> todoWorkItems_;
	std::vector<WorkItem> finishedWorkItems_;
	std::vector<PJWorkerThread*> workerThreads_;

	bool isWorking_, verify_, ignoreWarnings_, discardMetadata_, createDir_;
	size_t todoWorkItemsCount_, readFinishedWorkItemIndex_;
};

#endif
