/**
 * Copyright (C) 2014 Roman Hiestand
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this software
 * and associated documentation files (the "Software"), to deal in the Software without restriction,
 * including without limitation the rights to use, copy, modify, merge, publish, distribute,
 * sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all copies or substantial
 * portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
 * LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#ifndef PREVIEWGENERATORTHREAD_H
#define PREVIEWGENERATORTHREAD_H

class PreviewGeneratorThread : public wxThread
{
public:
	PreviewGeneratorThread();
	virtual ~PreviewGeneratorThread();

	// Starting/stopping the preview thread
	bool startPreviewThread();
	void stopPreviewThread();

	// Adding a new preview request
	void addPreviewRequest(const wxString &filename);

	// Ask for finished preview image
	bool getNewPreviewImage(wxImage &previewImage, wxString &previewFilename);

	// Cancel a running preview generation
	void cancelPreview();

protected:
	virtual wxThread::ExitCode Entry();

	bool createPreview(const wxString &localPreviewRequestFilename, wxImage &localPreviewImage);

private:
	wxMutex mutex_;
	wxCondition cond_;
	wxString previewRequestFilename_;
	bool hasNewPreviewRequest_;
	bool terminate_;
	bool cancelPreview_;

	wxMutex outputMutex_;
	wxImage previewImage_;
	wxString previewFilename_;
	bool hasNewPreview_;
};

#endif
