/**
 * Copyright (C) 2013 Roman Hiestand
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this software
 * and associated documentation files (the "Software"), to deal in the Software without restriction,
 * including without limitation the rights to use, copy, modify, merge, publish, distribute,
 * sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all copies or substantial
 * portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
 * LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

// wxPackJPGShellExt.cpp : Implementation of CwxPackJPGShellExt

#include "stdafx.h"
#include "wxPackJPGShellExt.h"

// From wxPackJPG
#include "../pjoledefs.h"

// Enable the check whether this DLL is used inside Windows Explorer
//#define WXP_CHECK_INSIDE_EXPLORER

// CwxPackJPGShellExt

CwxPackJPGShellExt::CwxPackJPGShellExt() :
	m_packJPGIconBmp(NULL)
{
	m_packJPGIconBmp = LoadBitmap ( _AtlBaseModule.GetModuleInstance(),
		MAKEINTRESOURCE(IDB_PACKJPGICONBMP) );
}

CwxPackJPGShellExt::~CwxPackJPGShellExt()
{
    if ( NULL != m_packJPGIconBmp )
       DeleteObject ( m_packJPGIconBmp );
}

HRESULT CwxPackJPGShellExt::Initialize (
	LPCITEMIDLIST pidlFolder, LPDATAOBJECT pDataObj, HKEY hProgID )
{
	TCHAR     szFile[MAX_PATH];
	UINT      uNumFiles;
	HDROP     hdrop;
	FORMATETC etc = { CF_HDROP, NULL, DVASPECT_CONTENT,
		-1, TYMED_HGLOBAL };
	STGMEDIUM stg = { TYMED_HGLOBAL };

#if defined(WXP_CHECK_INSIDE_EXPLORER)
	// Check whether we are running inside Windows explorer. If not,
	// don't show the context menu. Code inspired by TortoiseSVN.
	bool isProcessExplorer = true;
	static const WCHAR explorerEnvPath[]=_T("%SystemRoot%\\explorer.exe");
	WCHAR moduleName[MAX_PATH] = {0};
	WCHAR explorerPath[MAX_PATH] = {0};
	DWORD lenMN = GetModuleFileName(NULL, moduleName, _countof(moduleName));
	if(lenMN > 0)
	{
		DWORD lenEP = ExpandEnvironmentStrings(explorerEnvPath, explorerPath, _countof(explorerPath));
		if(lenEP > 0 && (lenEP <= _countof(explorerPath)))
		{
			isProcessExplorer = (lstrcmpi(moduleName, explorerPath) == 0);

			// we also have to allow the verclsid.exe process - that process determines
			// first whether the shell is allowed to even use an extension.
			if(!isProcessExplorer && (lenMN > 13))
			{
				if((lstrcmpi(&moduleName[lenMN-13], _T("\\verclsid.exe")) == 0))
					isProcessExplorer = true;
			}
		}
	}
	if(!isProcessExplorer)
		return E_UNEXPECTED;
#endif	// defined(WXP_CHECK_INSIDE_EXPLORER)

	// Read the list of folders from the data object.  They're stored in HDROP
	// form, so just get the HDROP handle and then use the drag 'n' drop APIs
	// on it.
	if ( FAILED( pDataObj->GetData ( &etc, &stg ) ))
		return E_INVALIDARG;

	// Get an HDROP handle.
	hdrop = (HDROP) GlobalLock ( stg.hGlobal );

	if ( NULL == hdrop )
	{
		ReleaseStgMedium ( &stg );
		return E_INVALIDARG;
	}

	// Clear list of files and other members
	m_lsFiles.clear();
	m_folderName.clear();
	m_nJPGFiles = m_nPJGFiles = m_nFolders = m_nIllegal = 0;

	// Determine how many files are involved in this operation.
	uNumFiles = DragQueryFile ( hdrop, 0xFFFFFFFF, NULL, 0 );

	for ( UINT uFile = 0; uFile < uNumFiles; uFile++ )
	{
		// Get the next filename.
		if ( 0 == DragQueryFile ( hdrop, uFile, szFile, MAX_PATH ) )
			continue;

		// Determine whether szFile is a file or a path
		BOOL isDir = PathIsDirectory(szFile);

		// Save folder name for later
		if(!isDir && m_folderName.empty())
		{
			TCHAR szFolderName[MAX_PATH];
			StringCchCopy(szFolderName, MAX_PATH, szFile);
			PathRemoveFileSpec(szFolderName);
			m_folderName = szFolderName;
		}

		// Determine whether szFile is a JPEG file
		BOOL isJPEG = PathIsContentType(szFile, _T("image/jpeg"));

		// Determine whether szFile is a Packed JPEG or JPEG file by ext
		BOOL isPJG = FALSE;
		PTSTR extString = PathFindExtension(szFile);
		if(extString != NULL && (*extString) != 0)
		{
			// Examine extension
			if(StrCmpNI(extString, _T(".pjg"), 4) == 0)
				isPJG = TRUE;
			else if(StrCmpNI(extString, _T(".jpg"), 4) == 0)
				isJPEG = TRUE;
			else if(StrCmpNI(extString, _T(".jpe"), 4) == 0)
				isJPEG = TRUE;
			else if(StrCmpNI(extString, _T(".jpeg"), 5) == 0)
				isJPEG = TRUE;
		}

		if(isDir)
			m_nFolders++;
		else if(isJPEG)
			m_nJPGFiles++;
		else if(isPJG)
			m_nPJGFiles++;
		else
			m_nIllegal++;

		m_lsFiles.push_back ( szFile );
	}   // end for

	// Release resources.
	GlobalUnlock ( stg.hGlobal );
	ReleaseStgMedium ( &stg );

	// If we found any files we can work with, return S_OK.  Otherwise,
	// return E_INVALIDARG so we don't get called again for this right-click
	// operation.
	bool isOK = false;
	isOK |= (m_nFolders == 1 && m_nJPGFiles == 0 && m_nPJGFiles == 0 && m_nIllegal == 0);
	isOK |= (m_nFolders == 0 && m_nJPGFiles > 0 && m_nPJGFiles == 0 && m_nIllegal == 0);
	isOK |= (m_nFolders == 0 && m_nJPGFiles == 0 && m_nPJGFiles > 0 && m_nIllegal == 0);
	return (isOK ? S_OK : E_INVALIDARG);
}


/////////////////////////////////////////////////////////////////////////////
// CwxPackJPGShellExt IContextMenu methods

//////////////////////////////////////////////////////////////////////////
//
// Function:    QueryContextMenu()
//
// Description:
//  Adds our items to the supplied menu.
//
//////////////////////////////////////////////////////////////////////////

HRESULT CwxPackJPGShellExt::QueryContextMenu (
	HMENU hmenu, UINT uMenuIndex, UINT uidFirstCmd, UINT uidLastCmd, UINT uFlags )
{
	// If the flags include CMF_DEFAULTONLY then we shouldn't do anything.
	if ( uFlags & CMF_DEFAULTONLY )
		return MAKE_HRESULT ( SEVERITY_SUCCESS, FACILITY_NULL, 0 );

	// First, create and populate a submenu.
	HMENU hSubmenu = CreatePopupMenu();
	UINT uID = uidFirstCmd;
	
	if(m_nFolders == 1)
	{
		// Create strings
		PTSTR pTempString = reinterpret_cast<PTSTR>(CoTaskMemAlloc( 1024 * sizeof(TCHAR) ));
		StringCchCopy(pTempString, 1024, _T("Compress all JPEG files to "));
		PTSTR pPathName = PathFindFileName(m_lsFiles.begin()->c_str());
		StringCchCat(pTempString, 1024, pPathName);
		StringCchCat(pTempString, 1024, _T("_compressed"));
		InsertMenu ( hSubmenu, 0, MF_BYPOSITION, uID++, pTempString );

		StringCchCopy(pTempString, 1024, _T("Uncompress all PJG files to "));
		pPathName = PathFindFileName(m_lsFiles.begin()->c_str());
		StringCchCat(pTempString, 1024, pPathName);
		StringCchCat(pTempString, 1024, _T("_uncompressed"));
		InsertMenu ( hSubmenu, 1, MF_BYPOSITION, uID++, pTempString );

		CoTaskMemFree( pTempString );
	}
	else if(m_nJPGFiles > 0)
	{
		InsertMenu ( hSubmenu, 0, MF_BYPOSITION, uID++, _T("Compress here") );
		InsertMenu ( hSubmenu, 1, MF_BYPOSITION, uID++, _T("Compress to \"compressed\" directory") );
	}
	else if(m_nPJGFiles > 0)
	{
		InsertMenu ( hSubmenu, 0, MF_BYPOSITION, uID++, _T("Uncompress here") );
		InsertMenu ( hSubmenu, 1, MF_BYPOSITION, uID++, _T("Uncompress to \"uncompressed\" directory") );
	}

	// Insert the submenu into the ctx menu provided by Explorer.
	MENUITEMINFO mii = { sizeof(MENUITEMINFO) };

	mii.fMask = MIIM_SUBMENU | MIIM_STRING | MIIM_ID;
	mii.wID = uID++;
	mii.hSubMenu = hSubmenu;
	mii.dwTypeData = _T("wxPackJPG");

	InsertMenuItem ( hmenu, uMenuIndex, TRUE, &mii );

	// Set the bitmap for the wxPackJPG item.
	if ( NULL != m_packJPGIconBmp )
		SetMenuItemBitmaps ( hmenu, uMenuIndex, MF_BYPOSITION, m_packJPGIconBmp, NULL );

	return MAKE_HRESULT ( SEVERITY_SUCCESS, FACILITY_NULL, uID - uidFirstCmd );
}


//////////////////////////////////////////////////////////////////////////
//
// Function:    GetCommandString()
//
// Description:
//  Sets the flyby help string for the Explorer status bar.
//
//////////////////////////////////////////////////////////////////////////

HRESULT CwxPackJPGShellExt::GetCommandString (
    UINT_PTR uCmdID, UINT uFlags, UINT* puReserved, LPSTR szName, UINT cchMax )
{
	USES_CONVERSION;
	LPCTSTR szPrompt;

	if ( uFlags & GCS_HELPTEXT )
	{
		switch ( uCmdID )
		{
		case 0:
			if(m_nFolders == 1)
				szPrompt = _T("Compress all JPEG files to new folder");
			else if(m_nJPGFiles > 0)
				szPrompt = _T("Compress here");
			else if(m_nPJGFiles > 0)
				szPrompt = _T("Uncompress here");
			break;
		case 1:
			if(m_nFolders == 1)
				szPrompt = _T("Uncompress all PJG files to new folder");
			else if(m_nJPGFiles > 0)
				szPrompt = _T("Compress to \"compressed\" directory");
			else if(m_nPJGFiles > 0)
				szPrompt = _T("Uncompress to \"uncompressed\" directory");
			break;
		case 2:
			szPrompt = _T("wxPackJPG");	// Submenu
			break;
		default:
			ATLASSERT(0);           // should never get here
			return E_INVALIDARG;
			break;
		}

		// Copy the help text into the supplied buffer.  If the shell wants
		// a Unicode string, we need to case szName to an LPCWSTR.
		if ( uFlags & GCS_UNICODE )
			lstrcpynW ( (LPWSTR) szName, T2CW(szPrompt), cchMax );
		else
			lstrcpynA ( szName, T2CA(szPrompt), cchMax );
	}
	else if ( uFlags & GCS_VERB )
	{
		switch ( uCmdID )
		{
		case 0:
			if(m_nFolders == 1)
				szPrompt = _T("wxPJCompressFolderToPJG");
			else if(m_nJPGFiles > 0)
				szPrompt = _T("wxPJCompressHere");
			else if(m_nPJGFiles > 0)
				szPrompt = _T("wxPJUncompressHere");
			break;
		case 1:
			if(m_nFolders == 1)
				szPrompt = _T("wxPJUncompressFolderToJPG");
			else if(m_nJPGFiles > 0)
				szPrompt = _T("wxPJCompressToFolder");
			else if(m_nPJGFiles > 0)
				szPrompt = _T("wxPJUncompressToFolder");
			break;
		case 2:
			szPrompt = _T("wxPackJPG");	// Submenu
			break;

		default:
			ATLASSERT(0);           // should never get here
			return E_INVALIDARG;
			break;
		}

		// Copy the help text into the supplied buffer.  If the shell wants
		// a Unicode string, we need to case szName to an LPCWSTR.
		if ( uFlags & GCS_UNICODE )
			lstrcpynW ( (LPWSTR) szName, T2CW(szPrompt), cchMax );
		else
			lstrcpynA ( szName, T2CA(szPrompt), cchMax );
	}

	return S_OK;
}


//////////////////////////////////////////////////////////////////////////
//
// Function:    InvokeCommand()
//
// Description:
//  Carries out the selected command.
//
// Notes on error handling:
//  hr is used as a return value for all COM methods. If any of the methods
//  fail, the loops/commands are aborted and the error will be returned
//  at the bottom of the method, to avoid leaks, i.e. do NOT immediately return!
//
//////////////////////////////////////////////////////////////////////////

HRESULT CwxPackJPGShellExt::InvokeCommand ( LPCMINVOKECOMMANDINFO pInfo )
{
	// If lpVerb really points to a string, ignore this function call and bail out.  
	if ( 0 != HIWORD( pInfo->lpVerb ) )
		return E_INVALIDARG;

	// Check that lpVerb is one of our commands
	int command = LOWORD(pInfo->lpVerb);
	switch ( command )
	{
	case 0:
	case 1:
		{
			IOleCommandTarget *pCmdTarget = NULL;
			HRESULT hr = CoCreateInstance(CLSID_WXPACKJPG, NULL,
				CLSCTX_LOCAL_SERVER, IID_IOleCommandTarget, (void **)&pCmdTarget);
			if(SUCCEEDED(hr))
			{
				// Bring wxPackJPG window to front
				CComVariant outParam;
				hr = pCmdTarget->Exec(NULL, wxPJCmd_Get_HWND,
					OLECMDIDF_WINDOWSTATE_USERVISIBLE | OLECMDEXECOPT_DODEFAULT,
					NULL, &outParam);
				if(SUCCEEDED(hr) && outParam.vt == VT_UI8)
				{
					BOOL isOK = ::SetForegroundWindow((HWND)outParam.ullVal);
					if(!isOK)
					{
						DWORD err = ::GetLastError();
						int jj = 27;
					}
				}

				if(m_nJPGFiles > 0 || m_nPJGFiles > 0)
				{
					// Send selected files to wxPackJPG via array of strings

					CComSafeArray<BSTR> sa;
					hr = sa.Create(static_cast<ULONG>(m_lsFiles.size()));
					if(FAILED(hr))
						return hr;

					string_list::const_iterator iter = m_lsFiles.begin();
					int pos = 0;
					while(iter != m_lsFiles.end() && SUCCEEDED(hr))
					{
						BSTR newBstr = SysAllocString(iter->c_str());
						hr = sa.SetAt(pos, newBstr, FALSE);
						if(FAILED(hr))
							return hr;

						iter++;
						pos++;
					}

					CComVariant stringArr(sa);
					hr = pCmdTarget->Exec(NULL, wxPJCmd_Transfer_Filename,
						OLECMDIDF_WINDOWSTATE_USERVISIBLE | OLECMDEXECOPT_DODEFAULT,
						&stringArr, NULL);

				}

				// Send command
				CComVariant folderParam(m_folderName.c_str());
				if(command == 0 && m_nJPGFiles > 0 && SUCCEEDED(hr))
				{
					hr = pCmdTarget->Exec(NULL, wxPJCmd_Compress_Here,
						OLECMDIDF_WINDOWSTATE_USERVISIBLE | OLECMDEXECOPT_DODEFAULT,
						NULL, NULL);
				}
				else if(command == 1 && m_nJPGFiles > 0 && SUCCEEDED(hr))
				{
					hr = pCmdTarget->Exec(NULL, wxPJCmd_Compress_To_Folder,
						OLECMDIDF_WINDOWSTATE_USERVISIBLE | OLECMDEXECOPT_DODEFAULT,
						&folderParam, NULL);
				}
				else if(command == 0 && m_nPJGFiles > 0 && SUCCEEDED(hr))
				{
					hr = pCmdTarget->Exec(NULL, wxPJCmd_Decompress_Here,
						OLECMDIDF_WINDOWSTATE_USERVISIBLE | OLECMDEXECOPT_DODEFAULT,
						NULL, NULL);
				}
				else if(command == 1 && m_nPJGFiles > 0 && SUCCEEDED(hr))
				{
					hr = pCmdTarget->Exec(NULL, wxPJCmd_Decompress_To_Folder,
						OLECMDIDF_WINDOWSTATE_USERVISIBLE | OLECMDEXECOPT_DODEFAULT,
						&folderParam, NULL);
				}
				else if(m_nFolders == 1)
				{
					CComVariant folderParam2(m_lsFiles.begin()->c_str());
					if(command == 0 && SUCCEEDED(hr))
					{
						hr = pCmdTarget->Exec(NULL, wxPJCmd_Compress_Folder_To_PJG,
							OLECMDIDF_WINDOWSTATE_USERVISIBLE | OLECMDEXECOPT_DODEFAULT,
							&folderParam2, NULL);
					}
					else if(command == 1 && SUCCEEDED(hr))
					{
						hr = pCmdTarget->Exec(NULL, wxPJCmd_Decompress_Folder_To_JPG,
							OLECMDIDF_WINDOWSTATE_USERVISIBLE | OLECMDEXECOPT_DODEFAULT,
							&folderParam2, NULL);
					}
				}

				pCmdTarget->Release();
			}
			else
				return E_INVALIDARG;

			return S_OK;
		}
		break;

	case 2:
		break;
	default:
		return E_FAIL;	// If the verb (...) is not recognized by the context menu handler, it must return
						// a failure (E_FAIL) to allow it to be passed on to the other context menu handlers that might implement that verb.
		break;
	}

	return S_OK;
}
