﻿; wxPackJPG.nsi
;
; This is an installer script for the Nullsoft Installer System (NSIS).
; See http://nsis.sourceforge.net/
;
;--------------------------------

; Use Modern UI 2
!include MUI2.nsh

!include "x64.nsh"
!include "Library.nsh"

;Get installation folder from registry if available
InstallDirRegKey HKCU "Software\hiesti.ch\wxPackJPG64" ""

!searchparse /file ..\version.h "#define WX_PACK_JPG_VERSION_MAJOR " VER_MAJOR
!searchparse /file ..\version.h `#define WX_PACK_JPG_VERSION_MINOR ` VER_MINOR
!searchparse /file ..\version.h `#define WX_PACK_JPG_VERSION_BUILD ` VER_BUILD
!searchparse /file ..\version.h '#define WX_PACK_JPG_NAME "' APP_NAME '"'
!searchparse /file ..\version.h '#define WX_PACK_JPG_COPYRIGHT_NAME "' COMPANY_NAME '"'
!searchparse /file ..\version.h '#define WX_PACK_JPG_COPYRIGHT_YEAR "' COPYRIGHT_YEAR '"'

; The name of the installer
Name "wxPackJPG ${VER_MAJOR}.${VER_MINOR}.${VER_BUILD}"

; The file to write
OutFile "wxPackJPG_${VER_MAJOR}.${VER_MINOR}.${VER_BUILD}_MinGW64_Setup.exe"

; The default installation directory
InstallDir $PROGRAMFILES64\wxPackJPG

; Request application privileges for Windows Vista
RequestExecutionLevel admin

XPStyle on

; Set best compression
SetCompressor lzma

; Set file and version information in the installer itself
VIAddVersionKey "ProductName" "wxPackJPG_Setup"
VIAddVersionKey "Comments" "Installer for ${APP_NAME} x64"
VIAddVersionKey "CompanyName" "${COMPANY_NAME}"
VIAddVersionKey "LegalCopyright" "Copyright (C) ${COPYRIGHT_YEAR} ${COMPANY_NAME}"
VIAddVersionKey "FileDescription" "Installer for ${APP_NAME}"
VIAddVersionKey "FileVersion" "${VER_MAJOR}.${VER_MINOR}.${VER_BUILD}"
VIProductVersion "${VER_MAJOR}.${VER_MINOR}.${VER_BUILD}.0"

; Do not automatically jump to the finish page, to allow the user to check the install log.
!define MUI_FINISHPAGE_NOAUTOCLOSE
!define MUI_UNFINISHPAGE_NOAUTOCLOSE
!define MUI_HEADERIMAGE
!define MUI_HEADERIMAGE_BITMAP "${NSISDIR}\Contrib\Graphics\Header\nsis.bmp"
!define MUI_HEADERIMAGE_UNBITMAP "${NSISDIR}\Contrib\Graphics\Header\nsis.bmp"

; Defines for the InstallLib/UnInstallLib macros
!define LIBRARY_X64
!define LIBRARY_SHELL_EXTENSION
!define LIBRARY_COM

Var StartMenuFolder
Var WXPSHDLL_ALREADY_INSTALLED
Var WXPJPGEXE_ALREADY_INSTALLED

;--------------------------------

; Pages

!insertmacro MUI_PAGE_LICENSE res\license.txt
!insertmacro MUI_PAGE_COMPONENTS
!insertmacro MUI_PAGE_DIRECTORY

;Start Menu Folder Page Configuration
!define MUI_STARTMENUPAGE_REGISTRY_ROOT "HKCU" 
!define MUI_STARTMENUPAGE_REGISTRY_KEY "Software\hiesti.ch\wxPackJPG64" 
!define MUI_STARTMENUPAGE_REGISTRY_VALUENAME "Start Menu Folder"
!define MUI_STARTMENUPAGE_DEFAULTFOLDER "wxPackJPG"
!insertmacro MUI_PAGE_STARTMENU Application $StartMenuFolder

!insertmacro MUI_PAGE_INSTFILES
!insertmacro MUI_PAGE_FINISH

!insertmacro MUI_UNPAGE_CONFIRM
!insertmacro MUI_UNPAGE_INSTFILES
!insertmacro MUI_UNPAGE_FINISH

!insertmacro MUI_LANGUAGE "English"
!insertmacro MUI_LANGUAGE "French"
!insertmacro MUI_LANGUAGE "German"
!insertmacro MUI_RESERVEFILE_LANGDLL

;--------------------------------
LangString Text_SelComplete ${LANG_ENGLISH} "Complete"
LangString Text_SelComplete ${LANG_GERMAN} "Komplett"
LangString Text_SelComplete ${LANG_FRENCH} "Complète"
LangString Text_SelMinimal ${LANG_ENGLISH} "Minimal"
LangString Text_SelMinimal ${LANG_GERMAN} "Minimal"
LangString Text_SelMinimal ${LANG_FRENCH} "Minimale"

InstType "$(Text_SelComplete)"
InstType "$(Text_SelMinimal)"

; The section for the main program
LangString Text_SectionMainProgram ${LANG_ENGLISH} "Main program"
LangString Text_SectionMainProgram ${LANG_GERMAN} "Hauptprogramm"
LangString Text_SectionMainProgram ${LANG_FRENCH} "Programme principal"
Section "$(Text_SectionMainProgram)" MainProgram

	SectionIn 1 2

	; Set output path to the installation directory.
	SetOutPath $INSTDIR

	; Store install path in registry
	WriteRegStr HKCU "Software\hiesti.ch\wxPackJPG64" "" $INSTDIR

	; Put files there (since wxPackJPG is a COM server, we use the InstallLib macro for registering)
	IfFileExists "$INSTDIR\wxPackJPG.exe" 0 new_installation_wxpjpgexe
		StrCpy $WXPJPGEXE_ALREADY_INSTALLED 1
new_installation_wxpjpgexe:
	!insertmacro InstallLib REGEXE $WXPJPGEXE_ALREADY_INSTALLED NOREBOOT_NOTPROTECTED ..\..\build_mingw64_msys\wxPackJPG.exe "$INSTDIR\wxPackJPG.exe" $TEMP
	WriteUninstaller "$INSTDIR\Uninstall.exe"

	; Insert links in start menu
	!insertmacro MUI_STARTMENU_WRITE_BEGIN Application
		CreateDirectory "$SMPROGRAMS\$StartMenuFolder"
		CreateShortCut "$SMPROGRAMS\$StartMenuFolder\wxPackJPG.lnk" "$INSTDIR\wxPackJPG.exe"
		CreateShortCut "$SMPROGRAMS\$StartMenuFolder\Uninstall.lnk" "$INSTDIR\Uninstall.exe"
  	!insertmacro MUI_STARTMENU_WRITE_END

	; Write uninstall routine and some additional info into registry
	WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\wxPackJPG64" "DisplayName" "wxPackJPG x64"
	WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\wxPackJPG64" "UninstallString" '"$INSTDIR\Uninstall.exe"'
	WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\wxPackJPG64" "QuietUninstallString" "$\"$INSTDIR\Uninstall.exe$\" /S"
	WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\wxPackJPG64" "InstallLocation" "$\"$INSTDIR$\""
	WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\wxPackJPG64" "DisplayIcon" "$\"$INSTDIR\wxPackJPG.exe$\""
	WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\wxPackJPG64" "Publisher" "${COMPANY_NAME}"
	WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\wxPackJPG64" "DisplayVersion" "${VER_MAJOR}.${VER_MINOR}.${VER_BUILD}"
	WriteRegDword HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\wxPackJPG64" "NoModify" "1"
	WriteRegDword HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\wxPackJPG64" "NoRepair" "1"

SectionEnd

LangString Text_SectionRegisterFiletype ${LANG_ENGLISH} "Register PJG file type"
LangString Text_SectionRegisterFiletype ${LANG_GERMAN} "Filetyp PJG registrieren"
LangString Text_SectionRegisterFiletype ${LANG_FRENCH} "Enregistrer type de fichier PJG"
Section "$(Text_SectionRegisterFiletype)" RegisterFiletype

	SectionIn 1
	
	WriteRegStr HKCR ".pjg" "" "wxPackJPG.PackedJPEG"

	WriteRegStr HKCR "wxPackJPG.PackedJPEG" "" `Packed JPEG`
	WriteRegStr HKCR "wxPackJPG.PackedJPEG\DefaultIcon" "" `"$INSTDIR\wxPackJPG.exe",1`
	WriteRegStr HKCR "wxPackJPG.PackedJPEG\shell" "" "open"
	WriteRegStr HKCR "wxPackJPG.PackedJPEG\shell\open" "" `Open with wxPackJPG`
	WriteRegStr HKCR "wxPackJPG.PackedJPEG\shell\open\DropTarget" "" ""
	WriteRegStr HKCR "wxPackJPG.PackedJPEG\shell\open\DropTarget" "Clsid" "{cf48bd7f-52ee-4b4a-85b6-12b31b69c35b}"

	; !defines for use with SHChangeNotify
	!ifdef SHCNE_ASSOCCHANGED
	!undef SHCNE_ASSOCCHANGED
	!endif
	!define SHCNE_ASSOCCHANGED 0x08000000
	!ifdef SHCNF_FLUSH
	!undef SHCNF_FLUSH
	!endif
	!define SHCNF_FLUSH        0x1000

	; Using the system.dll plugin to call the SHChangeNotify Win32 API function so we
	; can update the shell.
	System::Call "shell32::SHChangeNotify(i,i,i,i) (${SHCNE_ASSOCCHANGED}, ${SHCNF_FLUSH}, 0, 0)"

SectionEnd

LangString Text_SectionRegisterShellExtension ${LANG_ENGLISH} "Add Context Menu for Windows Explorer"
LangString Text_SectionRegisterShellExtension ${LANG_GERMAN} "Windows Explorer-Kontextmenü installieren"
LangString Text_SectionRegisterShellExtension ${LANG_FRENCH} "Installer un menu contextuel pour l'Explorateur Windows"
Section "$(Text_SectionRegisterShellExtension)" RegisterShellExtension

	SectionIn 1

	; Set output path to the installation directory.
	SetOutPath $INSTDIR

	IfFileExists "$INSTDIR\wxPackJPEGShell.dll" 0 new_installation_wxpshdll
		StrCpy $WXPSHDLL_ALREADY_INSTALLED 1
new_installation_wxpshdll:

	!insertmacro InstallLib REGDLL $WXPSHDLL_ALREADY_INSTALLED REBOOT_NOTPROTECTED ..\ShellExt\x64\Release\wxPackJPEGShell.dll "$INSTDIR\wxPackJPEGShell.dll" $TEMP

SectionEnd

LangString Text_MsgBox64BitWindowsOnly ${LANG_ENGLISH} "This installer only works on a 64 bit version of Windows."
LangString Text_MsgBox64BitWindowsOnly ${LANG_GERMAN} "Dieses Installationsprogramm funktioniert nur auf einer 64-Bit-Version von Windows"
LangString Text_MsgBox64BitWindowsOnly ${LANG_FRENCH} "Ce programme d'installation ne fonctionne que sur une version 64 bits de Windows"
Function .onInit
	${IfNot} ${RunningX64}
		MessageBox MB_OK "$(Text_MsgBox64BitWindowsOnly)"
		Abort
	${EndIf}
	!insertmacro MUI_LANGDLL_DISPLAY
FunctionEnd

Section "Uninstall" Uninstall

	; Unregister wxPackJPEGShell.dll
	!insertmacro UnInstallLib REGDLL NOTSHARED REBOOT_NOTPROTECTED "$INSTDIR\wxPackJPEGShell.dll"

	DeleteRegKey HKCR "wxPackJPG.PackedJPEG"

	Delete "$INSTDIR\Uninstall.exe"
	!insertmacro UnInstallLib REGEXE NOTSHARED NOREBOOT_NOTPROTECTED "$INSTDIR\wxPackJPG.exe"
	RMDir "$INSTDIR"

	!insertmacro MUI_STARTMENU_GETFOLDER Application $StartMenuFolder
	Delete "$SMPROGRAMS\$StartMenuFolder\Uninstall.lnk"
	Delete "$SMPROGRAMS\$StartMenuFolder\wxPackJPG.lnk"
	RMDir "$SMPROGRAMS\$StartMenuFolder"
	DeleteRegKey /ifempty HKCU "Software\hiesti.ch\wxPackJPG64"
	DeleteRegKey HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\wxPackJPG64"

SectionEnd

Function un.onInit
  !insertmacro MUI_UNGETLANGUAGE
FunctionEnd

LangString DESC_MainProgram ${LANG_ENGLISH} "The main program."
LangString DESC_RegisterFiletype ${LANG_ENGLISH} "Link .PJG extension to wxPackJPG."
LangString DESC_RegisterShellExtension ${LANG_ENGLISH} "Add a wxPackJPG context menu for JPG and PJG files."
LangString DESC_MainProgram ${LANG_GERMAN} "Das Hauptprogramm."
LangString DESC_RegisterFiletype ${LANG_GERMAN} "Verknüpfe die Endung .PJG mit wxPackJPG."
LangString DESC_RegisterShellExtension ${LANG_GERMAN} "Installiere ein wxPackJPG-Kontextmenü für JPG- und PJG-Dateien."
LangString DESC_MainProgram ${LANG_FRENCH} "Le programme principal."
LangString DESC_RegisterFiletype ${LANG_FRENCH} "Lièr des fichiers avec extension .PJG avec wxPackJPG."
LangString DESC_RegisterShellExtension ${LANG_FRENCH} "Installer un menu contextuel wxPackJPG pour des fichiers JPG et PJG."

!insertmacro MUI_FUNCTION_DESCRIPTION_BEGIN
	!insertmacro MUI_DESCRIPTION_TEXT ${MainProgram} $(DESC_MainProgram)
	!insertmacro MUI_DESCRIPTION_TEXT ${RegisterFiletype} $(DESC_RegisterFiletype)
	!insertmacro MUI_DESCRIPTION_TEXT ${RegisterShellExtension} $(DESC_RegisterShellExtension)
!insertmacro MUI_FUNCTION_DESCRIPTION_END
