/**
 * Copyright (C) 2014 Roman Hiestand
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this software
 * and associated documentation files (the "Software"), to deal in the Software without restriction,
 * including without limitation the rights to use, copy, modify, merge, publish, distribute,
 * sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all copies or substantial
 * portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
 * LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#include "CommonIncludes.h"

#include "wxPackJPGApp.h"
#include "wxPackJPGMainFrame.h"
#include <wx/cmdline.h>
#if defined(PJ_WIN32)
#	include "PJDropTargetWin32.h"
#endif

// Main program equivalent, creating windows and returning main app frame
bool wxPackJPGApp::OnInit()
{
	wxInitAllImageHandlers();

	// Parse command line
	static const wxCmdLineEntryDesc cmdLineDesc[] =
	{
#if defined(PJ_WIN32)
#	if wxCHECK_VERSION(2, 9, 0)
		{ wxCMD_LINE_SWITCH, "Embedding", "Embedding", "Started by COM/OLE", wxCMD_LINE_VAL_STRING, wxCMD_LINE_PARAM_OPTIONAL },	// Only for Windows, called from COM/OLE (Explorer)
		{ wxCMD_LINE_SWITCH, "regserver", "RegServer", "Register as COM Server", wxCMD_LINE_VAL_STRING, wxCMD_LINE_PARAM_OPTIONAL },	// Only for Windows
		{ wxCMD_LINE_SWITCH, "unregserver", "UnRegserver", "Unregister as COM Server", wxCMD_LINE_VAL_STRING, wxCMD_LINE_PARAM_OPTIONAL },	// Only for Windows
#	else
		{ wxCMD_LINE_SWITCH, wxT("Embedding"), wxT("Embedding"), wxT("Started by COM/OLE"), wxCMD_LINE_VAL_STRING, wxCMD_LINE_PARAM_OPTIONAL },	// Only for Windows, called from COM/OLE (Explorer)
		{ wxCMD_LINE_SWITCH, wxT("regserver"), wxT("RegServer"), wxT("Register as COM Server"), wxCMD_LINE_VAL_STRING, wxCMD_LINE_PARAM_OPTIONAL },	// Only for Windows
		{ wxCMD_LINE_SWITCH, wxT("unregserver"), wxT("UnRegserver"), wxT("Unregister as COM Server"), wxCMD_LINE_VAL_STRING, wxCMD_LINE_PARAM_OPTIONAL },	// Only for Windows
#	endif
#endif
		{ wxCMD_LINE_NONE }
	};

	bool calledFromCOM_OLE = false;
	bool regServer = false, unregServer = false;
	wxCmdLineParser parser(argc, argv);
	parser.SetDesc(cmdLineDesc);

	if(parser.Parse(true) == 0)
	{
#if defined(PJ_WIN32)
		if(parser.Found(wxT("Embedding")))
			calledFromCOM_OLE = true;
		if(parser.Found(wxT("regserver")))
			regServer = true;
		if(parser.Found(wxT("unregserver")))
			unregServer = true;
#endif
		/* For later: Parse file names given via command line
		size_t ct = parser.GetParamCount();
		for(size_t i = 0; i < ct; i++)
		{
			wxString paramI = parser.GetParam(i);
		}*/
	}

#if defined(PJ_WIN32)
	if(regServer)
	{
		PJDropTargetWin32::registerServer();
		return false;	// Don't open the main window
	}
	if(unregServer)
	{
		PJDropTargetWin32::unregisterServer();
		return false;	// Don't open the main window
	}
#endif

	pMainFrame_ = new wxPackJPGMainFrame(NULL, calledFromCOM_OLE);

	// Show the frame
	pMainFrame_->Show(true);

	SetTopWindow(pMainFrame_);

	return true;
}


IMPLEMENT_APP(wxPackJPGApp)
