/**
 * Copyright (C) 2014 Roman Hiestand
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this software
 * and associated documentation files (the "Software"), to deal in the Software without restriction,
 * including without limitation the rights to use, copy, modify, merge, publish, distribute,
 * sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all copies or substantial
 * portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
 * LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#ifndef WXPACKJPGMAINFRAME_H
#define WXPACKJPGMAINFRAME_H

class PJDropTarget;
class PJDropTargetWin32;
class PJWorkerThreadManager;

#include "wxPackJPGMainFrameBase.h"
#include "pjoledefs.h"
#include "PJDirTreeThread.h"
#include "PreviewGeneratorThread.h"

class wxPackJPGMainFrame: public wxPackJPGMainFrameBase
{
public:
	wxPackJPGMainFrame(wxWindow* parent, bool calledFromCOM_OLE);
	virtual ~wxPackJPGMainFrame();

	bool addDragAndDropFiles(const wxArrayString &filenames);
	bool addShellExtFiles(const wxArrayString &filenames);
	bool addFileToCompressList(const wxString &filename);
	bool addFileToUncompressList(const wxString &filename);
	void removeSelectedItems(wxListCtrl *pListCtrl);
	void addItemsToThreadManager(wxListCtrl *pListCtrl, bool outputToPath,
		bool overwriteFiles, bool verify,
		const wxString &outPath, const wxString &newFileExt);

	void enableUI(bool enable);
	void startCommand(wxPackJPG_OLE_Commands cmd, const wxString &pathStr);

protected:
	virtual void OnMainFrameClose( wxCloseEvent& event );
	virtual void OnExitMenuItem( wxCommandEvent& event );
	virtual void OnAboutMenuItem( wxCommandEvent& event );
	virtual void OnMainNotebookPageChanging( wxNotebookEvent& event );
	virtual void OnCompressButton( wxCommandEvent& event );
	virtual void OnAbortCompressionButton( wxCommandEvent& event );
	virtual void OnSaveCompressedFilesChoice( wxCommandEvent& event );
	virtual void OnAddFilesButton( wxCommandEvent& event );
	virtual void OnRemoveSelectedFilesButton( wxCommandEvent& event );
	virtual void OnClearFileListButton( wxCommandEvent& event );
	virtual void OnCompressFileListColClick( wxListEvent& event );
	virtual void OnCompressFileListItemDeselected( wxListEvent& event );
	virtual void OnCompressFileListItemSelected( wxListEvent& event );
	virtual void OnCompressFileListKeyDown( wxListEvent& event );
	virtual void OnDecompressButton( wxCommandEvent& event );
	virtual void OnAbortDecompressionButton( wxCommandEvent& event );
	virtual void OnSaveUncompressedFilesChoice( wxCommandEvent& event );
	virtual void OnUCAddFilesButton( wxCommandEvent& event );
	virtual void OnUCRemoveSelectedFilesButton( wxCommandEvent& event );
	virtual void OnUCClearFileListButton( wxCommandEvent& event );
	virtual void OnUCompressFileListColClick( wxListEvent& event );
	virtual void OnUCompressFileListItemDeselected( wxListEvent& event );
	virtual void OnUCompressFileListItemSelected( wxListEvent& event );
	virtual void OnUCompressFileListKeyDown( wxListEvent& event );

	virtual void OnShowFileTypeWarningJPEG( wxCommandEvent &event );
	virtual void OnShowFileTypeWarningPJG( wxCommandEvent &event );
	virtual void OnShowProcessingErrors( wxCommandEvent &event );
	virtual void OnCmdCompressFolderToPJG( wxCommandEvent &event );
	virtual void OnCmdDecompressFolderToJPG( wxCommandEvent &event );
	virtual void OnStartToFolder( wxCommandEvent &event );
	virtual void OnTimer( wxTimerEvent &event );

#if wxCHECK_VERSION(2, 9, 0)
	static int wxCALLBACK wxListCompareFunction(wxIntPtr item1, wxIntPtr item2, wxIntPtr sortData);
#else
	static int wxCALLBACK wxListCompareFunction(long item1, long item2, long sortData);
#endif
	wxIcon getIcon();
	void enableSizer(wxSizer *pSizer, bool enable, bool recursive);

private:
	wxTimer aTimer_;

	PJDropTarget *pDropTarget_;
	PJDropTargetWin32 *pPJDropTargetWin32_;
	PJWorkerThreadManager *pPJWorkerThreadManager_;

	int nextListID_[2];

	int failedItemsCount_, warningItemsCount_;

	PJDirTreeThread *pPJDirTreeThread_;
	std::list< PJDirTreeThread::FoundFile > dirTreeFiles_;

	bool firstTimeOnTimer_;
	bool calledFromCOM_OLE_;	// Only for Windows: Enable COM/OLE server
	bool closeAfterCommandFinished_;

	// Parameters for column sorting
	static int sortColumn_;
	static wxListCtrl *pSortListCtrl_;
	static std::vector<size_t> sortFileSizesC_, sortFileSizesUC_, *pSortFileSizes_;
	static int sortDirections_[4], sortDirectionOffset_;

	PreviewGeneratorThread previewGeneratorThread_;

	DECLARE_EVENT_TABLE()
};

#endif
